import 'package:flutter/material.dart';
import 'package:flutter_reorderable_grid_view/widgets/widgets.dart';

void main() {
  runApp(const MaterialApp(home: MyApp()));
}

class MyApp extends StatefulWidget {
  const MyApp({super.key});

  @override
  State<MyApp> createState() => _MyAppState();
}

class _MyAppState extends State<MyApp> {
  final _scrollController = ScrollController();
  final _gridViewKey = GlobalKey();

  var _fruits = <String>["apple", "banana", "strawberry"];

  @override
  Widget build(BuildContext context) {
    final generatedChildren = List.generate(
      _fruits.length,
      (index) => InkWell(
        key: Key(_fruits.elementAt(index)),
        onTap: () {
          debugPrint('hello');
        },
        child: ColoredBox(
          color: Colors.lightBlue,
          child: Text(
            _fruits.elementAt(index),
          ),
        ),
      ),
    );

    return Scaffold(
      body: ReorderableBuilder(
        scrollController: _scrollController,
        onReorder: (ReorderedListFunction<String> reorderedListFunction) {
          setState(() {
            _fruits = reorderedListFunction(_fruits);
          });
        },
        children: generatedChildren,
        builder: (children) {
          return GridView(
            key: _gridViewKey,
            controller: _scrollController,
            gridDelegate: const SliverGridDelegateWithFixedCrossAxisCount(
              crossAxisCount: 4,
              mainAxisSpacing: 4,
              crossAxisSpacing: 8,
            ),
            children: children,
          );
        },
      ),
    );
  }
}
